@@ -1,40 +1,17 @@
 #!/bin/bash
-# Copyright 1999-2003 Gentoo Technologies, Inc.
+# Copyright 1999-2004 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Header: /home/cvsroot/gentoo-src/portage/bin/ebuild.sh,v 1.154 2004/02/04 22:40:21 carpaski Exp $
+# $Header: /var/cvsroot/gentoo-src/portage/bin/ebuild.sh,v 1.201.2.6 2004/12/03 04:15:29 carpaski Exp $
 
-SANDBOX_PREDICT="${SANDBOX_PREDICT}:/proc/self/maps:/dev/console:/usr/lib/portage/pym"
-SANDBOX_WRITE="${SANDBOX_WRITE}:/dev/shm:${PORTAGE_TMPDIR}"
-SANDBOX_READ="${SANDBOX_READ}:/dev/shm:${PORTAGE_TMPDIR}"
+export SANDBOX_PREDICT="${SANDBOX_PREDICT}:/proc/self/maps:/dev/console:/usr/lib/portage/pym:/dev/random"
+export SANDBOX_WRITE="${SANDBOX_WRITE}:/dev/shm:${PORTAGE_TMPDIR}"
+export SANDBOX_READ="${SANDBOX_READ}:/dev/shm:${PORTAGE_TMPDIR}"
 
+if [ ! -z "${PORTAGE_GPG_DIR}" ]; then
+	SANDBOX_PREDICT="${SANDBOX_PREDICT}:${PORTAGE_GPG_DIR}"
+fi
 
 if [ "$*" != "depend" ] && [ "$*" != "clean" ] && [ "$*" != "nofetch" ]; then
-	if [ -f "${T}/successful" ]; then
-		rm -f "${T}/successful"
-	fi
-
-	# Hurray for per-ebuild logging.
-	if [ ! -z "${PORT_LOGDIR}" ]; then
-		if [ -z "${PORT_LOGGING}" ]; then
-			export PORT_LOGGING=1
-			export SANDBOX_WRITE="$SANDBOX_WRITE:${PORT_LOGDIR}"
-			install -d "${PORT_LOGDIR}" &>/dev/null
-			chown root:portage "${PORT_LOGDIR}" &>/dev/null
-			chmod g+rwxs "${PORT_LOGDIR}" &> /dev/null
-			touch "${PORT_LOGDIR}/${LOG_COUNTER}-${PF}.log" &> /dev/null
-			chmod g+w "${PORT_LOGDIR}/${LOG_COUNTER}-${PF}.log" &> /dev/null
-			echo "$*" >> "${PORT_LOGDIR}/${LOG_COUNTER}-${PF}.log"
-			{ $0 $* || rm -f "${T}/successful" ; } 2>&1 \
-			  | tee -i -a "${PORT_LOGDIR}/${LOG_COUNTER}-${PF}.log"
-			if [ -f "${T}/successful" ]; then
-				rm -f "${T}/successful"
-				exit 0
-			else
-				exit 1
-			fi
-		fi
-	fi
-
 	if [ -f "${T}/environment" ]; then
 		source "${T}/environment" &>/dev/null
 	fi
@@ -44,33 +21,37 @@ if [ -n "$#" ]; then
 	ARGS="${*}"
 fi
 
+declare -rx EBUILD_PHASE="$*"
+
 # Prevent aliases from causing portage to act inappropriately.
 # Make sure it's before everything so we don't mess aliases that follow.
 unalias -a
 
 # Unset some variables that break things.
-unset GZIP BZIP BZIP2 CDPATH GREP_OPTIONS GREP_COLOR
+unset GZIP BZIP BZIP2 CDPATH GREP_OPTIONS GREP_COLOR GLOBIGNORE
 
-# We need this next line for "die" and "assert". It expands 
+# We need this next line for "die" and "assert". It expands
 # It _must_ preceed all the calls to die and assert.
 shopt -s expand_aliases
 alias die='diefunc "$FUNCNAME" "$LINENO" "$?"'
 alias assert='_pipestatus="${PIPESTATUS[*]}"; [[ "${_pipestatus// /}" -eq 0 ]] || diefunc "$FUNCNAME" "$LINENO" "$_pipestatus"'
+alias save_IFS='[ "${IFS:-unset}" != "unset" ] && old_IFS="${IFS}"'
+alias restore_IFS='if [ "${old_IFS:-unset}" != "unset" ]; then IFS="${old_IFS}"; unset old_IFS; else unset IFS; fi'
 
 OCC="$CC"
 OCXX="$CXX"
 if [ "$USERLAND" == "GNU" ]; then
 	source /etc/profile.env &>/dev/null
 fi
+if [ -f "${PORTAGE_BASHRC}" ]; then
+	source "${PORTAGE_BASHRC}"
+fi
 [ ! -z "$OCC" ] && export CC="$OCC"
 [ ! -z "$OCXX" ] && export CXX="$OCXX"
 
 export PATH="/sbin:/usr/sbin:/usr/lib/portage/bin:/bin:/usr/bin:${ROOTPATH}"
 [ ! -z "$PREROOTPATH" ] && export PATH="${PREROOTPATH%%:}:$PATH"
 
-# Grab our new utility functions.
-source /usr/lib/portage/bin/extra_functions.sh
-
 if [ -e /etc/init.d/functions.sh ]; then
 	source /etc/init.d/functions.sh  &>/dev/null
 elif [ -e /etc/rc.d/config/functions ];	then
@@ -80,6 +61,42 @@ else
 	source /usr/lib/portage/bin/functions.sh &>/dev/null
 fi
 
+# the sandbox is disabled by default except when overridden in the relevant stages
+export SANDBOX_ON="0"
+
+# sandbox support functions; defined prior to profile.bashrc srcing, since the profile might need to add a default exception (/usr/lib64/conftest fex, bug #60147)
+addread()
+{
+	export SANDBOX_READ="$SANDBOX_READ:$1"
+}
+
+addwrite()
+{
+	export SANDBOX_WRITE="$SANDBOX_WRITE:$1"
+}
+
+adddeny()
+{
+	export SANDBOX_DENY="$SANDBOX_DENY:$1"
+}
+
+addpredict()
+{
+	export SANDBOX_PREDICT="$SANDBOX_PREDICT:$1"
+}
+
+
+# source the existing profile.bashrc's.
+save_IFS
+IFS=$'\n'
+for dir in ${PROFILE_PATHS}; do
+	if [ -f "${dir}/profile.bashrc" ]; then
+		source "${dir}/profile.bashrc"
+	fi
+done
+restore_IFS
+
+
 esyslog() {
 	# Custom version of esyslog() to take care of the "Red Star" bug.
 	# MUST follow functions.sh to override the "" parameter problem.
@@ -88,6 +105,13 @@ esyslog() {
 
 
 use() {
+	if useq ${1}; then
+		return 0
+	fi
+	return 1
+}
+
+usev() {
 	if useq ${1}; then
 		echo "${1}"
 		return 0
@@ -103,6 +127,12 @@ useq() {
 		neg=1
 	fi
 	local x
+	
+	# Make sure we have this USE flag in IUSE
+	if ! hasq "${u}" ${IUSE} ${E_IUSE} && ! hasq "${u}" ${PORTAGE_ARCHLIST} selinux; then
+		echo "QA Notice: USE Flag '${u}' not in IUSE for ${CATEGORY}/${PF}" >&2
+	fi
+
 	for x in ${USE}; do
 		if [ "${x}" == "${u}" ]; then
 			if [ ${neg} -eq 1 ]; then
@@ -120,6 +150,13 @@ useq() {
 }
 
 has() {
+	if hasq "$@"; then
+		return 0
+	fi
+	return 1
+}
+
+hasv() {
 	if hasq "$@"; then
 		echo "${1}"
 		return 0
@@ -144,7 +181,14 @@ hasq() {
 }
 
 has_version() {
-	[ "${EBUILD_PHASE}" == "depend" ] && echo "has_version() in global scope: ${CATEGORY}/$PF" >&2
+	if [ "${EBUILD_PHASE}" == "depend" ]; then
+		echo -n "QA Notice: has_version() in global scope: " >&2
+		if [ ${ECLASS_DEPTH} -gt 0 ]; then
+			echo "eclass ${ECLASS}" >&2
+		else
+			echo "${CATEGORY}/${PF}" >&2
+		fi
+	fi
 	# return shell-true/shell-false if exists.
 	# Takes single depend-type atoms.
 	if /usr/lib/portage/bin/portageq 'has_version' "${ROOT}" "$1"; then
@@ -155,12 +199,32 @@ has_version() {
 }
 
 portageq() {
-	[ "${EBUILD_PHASE}" == "depend" ] && echo "portageq in global scope: ${CATEGORY}/$PF" >&2
+	if [ "${EBUILD_PHASE}" == "depend" ]; then
+		echo -n "QA Notice: portageq in global scope: " >&2
+		if [ ${ECLASS_DEPTH} -gt 0 ]; then
+			echo "eclass ${ECLASS}" >&2
+		else
+			echo "${CATEGORY}/${PF}" >&2
+		fi
+	fi
 	/usr/lib/portage/bin/portageq "$@"
 }
 
+
+# ----------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
+
+
 best_version() {
-	[ "${EBUILD_PHASE}" == "depend" ] && echo "best_version() in global scope: ${CATEGORY}/$PF" >&2
+	if [ "${EBUILD_PHASE}" == "depend" ]; then
+		echo -n "QA Notice: best_version() in global scope: " >&2
+		if [ ${ECLASS_DEPTH} -gt 0 ]; then
+			echo "eclass ${ECLASS}" >&2
+		else
+			echo "${CATEGORY}/${PF}" >&2
+		fi
+	fi
 	# returns the best/most-current match.
 	# Takes single depend-type atoms.
 	/usr/lib/portage/bin/portageq 'best_version' "${ROOT}" "$1"
@@ -170,7 +234,7 @@ use_with() {
 	if [ -z "$1" ]; then
 		echo "!!! use_with() called without a parameter." >&2
 		echo "!!! use_with <USEFLAG> [<flagname> [value]]" >&2
-		return
+		return 1
 	fi
 
 	local UW_SUFFIX=""	
@@ -185,18 +249,17 @@ use_with() {
 	
 	if useq $1; then
 		echo "--with-${UWORD}${UW_SUFFIX}"
-		return 0
 	else
 		echo "--without-${UWORD}"
-		return 1
 	fi
+	return 0
 }
 
 use_enable() {
 	if [ -z "$1" ]; then
 		echo "!!! use_enable() called without a parameter." >&2
 		echo "!!! use_enable <USEFLAG> [<flagname> [value]]" >&2
-		return
+		return 1
 	fi
 
 	local UE_SUFFIX=""	
@@ -211,11 +274,10 @@ use_enable() {
 	
 	if useq $1; then
 		echo "--enable-${UWORD}${UE_SUFFIX}"
-		return 0
 	else
 		echo "--disable-${UWORD}"
-		return 1
 	fi
+	return 0
 }
 
 diefunc() {
@@ -225,6 +287,7 @@ diefunc() {
 	echo "!!! ERROR: $CATEGORY/$PF failed." >&2
 	echo "!!! Function $funcname, Line $lineno, Exitcode $exitcode" >&2
 	echo "!!! ${*:-(no error message)}" >&2
+	echo "!!! If you need support, post the topmost build error, NOT this status message." >&2
 	echo >&2
 	exit 1
 }
@@ -272,31 +335,6 @@ keepdir()
 	fi
 }
 
-# the sandbox is disabled by default except when overridden in the relevant stages
-export SANDBOX_ON="0"
-
-# sandbox support functions
-addread()
-{
-	export SANDBOX_READ="$SANDBOX_READ:$1"
-}
-
-addwrite()
-{
-	export SANDBOX_WRITE="$SANDBOX_WRITE:$1"
-}
-
-adddeny()
-{
-	export SANDBOX_DENY="$SANDBOX_DENY:$1"
-}
-
-addpredict()
-{
-	export SANDBOX_PREDICT="$SANDBOX_PREDICT:$1"
-}
-
-
 unpack() {
 	local x
 	local y
@@ -312,31 +350,32 @@ unpack() {
 	for x in "$@"; do
 		myfail="failure unpacking ${x}"
 		echo ">>> Unpacking ${x} to $(pwd)"
-		y="$(echo $x | sed 's:.*\.\(tar\)\.[a-zA-Z0-9]*:\1:')"
+		y="${x%.*}"
+		y="${y##*.}"
 
 		case "${x##*.}" in
-			tar) 
-				tar ${tarvars} -xf "${DISTDIR}/${x}" || die "$myfail"
+			tar)
+				tar xf "${DISTDIR}/${x}" ${tarvars} || die "$myfail"
 				;;
-			tgz) 
-				tar ${tarvars} -xzf "${DISTDIR}/${x}" || die "$myfail"
+			tgz)
+				tar xzf "${DISTDIR}/${x}" ${tarvars} || die "$myfail"
 				;;
-			tbz2) 
-				bzip2 -dc "${DISTDIR}/${x}" | tar ${tarvars} -xf - || die "$myfail"
+			tbz2)
+				bzip2 -dc "${DISTDIR}/${x}" | tar xf - ${tarvars} || die "$myfail"
 				;;
-			ZIP|zip) 
+			ZIP|zip)
 				unzip -qo "${DISTDIR}/${x}" || die "$myfail"
 				;;
-			gz|Z|z) 
+			gz|Z|z)
 				if [ "${y}" == "tar" ]; then
-					tar ${tarvars} -xzf "${DISTDIR}/${x}" || die "$myfail"
+					tar xzf "${DISTDIR}/${x}" ${tarvars} || die "$myfail"
 				else
 					gzip -dc "${DISTDIR}/${x}" > ${x%.*} || die "$myfail"
 				fi
 				;;
-			bz2) 
+			bz2)
 				if [ "${y}" == "tar" ]; then
-					bzip2 -dc "${DISTDIR}/${x}" | tar ${tarvars} -xf - || die "$myfail"
+					bzip2 -dc "${DISTDIR}/${x}" | tar xf - ${tarvars} || die "$myfail"
 				else
 					bzip2 -dc "${DISTDIR}/${x}" > ${x%.*} || die "$myfail"
 				fi
@@ -349,43 +388,92 @@ unpack() {
 }
 
 econf() {
-	if [ -x ./configure ]; then
+	if [ -z "${ECONF_SOURCE}" ]; then
+		ECONF_SOURCE="."
+	fi
+	if [ -x "${ECONF_SOURCE}/configure" ]; then
+		if hasq autoconfig $FEATURES && ! hasq autoconfig $RESTRICT; then
+			if [ -e /usr/share/gnuconfig/ -a -x /bin/basename ]; then
+				local x
+				for x in $(find ${S} -type f -name config.guess -o -name config.sub) ; do
+					einfo "econf: updating $x with /usr/share/gnuconfig/$(/bin/basename ${x})"
+					cp /usr/share/gnuconfig/$(/bin/basename ${x}) ${x}
+				done
+			fi
+		fi
+
 		if [ ! -z "${CBUILD}" ]; then
 			EXTRA_ECONF="--build=${CBUILD} ${EXTRA_ECONF}"
 		fi
-		./configure \
-		    --prefix=/usr \
-		    --host=${CHOST} \
-		    --mandir=/usr/share/man \
-		    --infodir=/usr/share/info \
-		    --datadir=/usr/share \
-		    --sysconfdir=/etc \
-		    --localstatedir=/var/lib \
-				${EXTRA_ECONF} \
-		    "$@" || die "econf failed" 
+
+		if [ ! -z "${CTARGET}" ]; then
+			EXTRA_ECONF="--target=${CTARGET} ${EXTRA_ECONF}"
+		fi
+		
+		# if the profile defines a location to install libs to aside from default, pass it on.
+		# if the ebuild passes in --libdir, they're responsible for the conf_libdir fun.
+		if [ ! -z "${CONF_LIBDIR}" ] && [ "${*/--libdir}" == "$*" ]; then
+			if [ "${*/--prefix}" == "$*" ]; then
+				CONF_PREFIX="/usr"
+			else
+				local args="$(echo $*)"
+				local -a pref=($(echo ${args/*--prefix[= ]}))
+				CONF_PREFIX=${pref}
+			fi
+			export CONF_PREFIX
+			EXTRA_ECONF="--libdir=/${CONF_PREFIX}/${CONF_LIBDIR} ${EXTRA_ECONF}"
+		fi
+		
+		echo "${ECONF_SOURCE}/configure" \
+			--prefix=/usr \
+			--host=${CHOST} \
+			--mandir=/usr/share/man \
+			--infodir=/usr/share/info \
+			--datadir=/usr/share \
+			--sysconfdir=/etc \
+			--localstatedir=/var/lib \
+			${EXTRA_ECONF} \
+			"$@"
+
+		"${ECONF_SOURCE}/configure" \
+			--prefix=/usr \
+			--host=${CHOST} \
+			--mandir=/usr/share/man \
+			--infodir=/usr/share/info \
+			--datadir=/usr/share \
+			--sysconfdir=/etc \
+			--localstatedir=/var/lib \
+			${EXTRA_ECONF} \
+			"$@" || die "econf failed"
 	else
 		die "no configure script found"
 	fi
 }
 
 einstall() {
+	# CONF_PREFIX is only set if they didn't pass in libdir above.
+	if [ ! -z "${CONF_LIBDIR}" ] && [ "${CONF_PREFIX:-unset}" != "unset" ]; then
+		EXTRA_EINSTALL="libdir=${D}/${CONF_PREFIX}/${CONF_LIBDIR} ${EXTRA_EINSTALL}"
+	fi
 	if [ -f ./[mM]akefile -o -f ./GNUmakefile ] ; then
 		if [ ! -z "${PORTAGE_DEBUG}" ]; then
 			make -n prefix=${D}/usr \
-			    datadir=${D}/usr/share \
-			    infodir=${D}/usr/share/info \
-		  	  localstatedir=${D}/var/lib \
-			    mandir=${D}/usr/share/man \
-			    sysconfdir=${D}/etc \
-			    "$@" install
+				datadir=${D}/usr/share \
+				infodir=${D}/usr/share/info \
+				localstatedir=${D}/var/lib \
+				mandir=${D}/usr/share/man \
+				sysconfdir=${D}/etc \
+				${EXTRA_EINSTALL} \
+				"$@" install
 		fi
 		make prefix=${D}/usr \
-		    datadir=${D}/usr/share \
-		    infodir=${D}/usr/share/info \
-		    localstatedir=${D}/var/lib \
-		    mandir=${D}/usr/share/man \
-		    sysconfdir=${D}/etc \
-		    "$@" install || die "einstall failed" 
+			datadir=${D}/usr/share \
+			infodir=${D}/usr/share/info \
+			localstatedir=${D}/var/lib \
+			mandir=${D}/usr/share/man \
+			sysconfdir=${D}/etc \
+			${EXTRA_EINSTALL} \
+			"$@" install || die "einstall failed"
 	else
 		die "no Makefile found"
 	fi
@@ -393,7 +481,7 @@ einstall() {
 
 pkg_setup()
 {
-	return 
+	return
 }
 
 pkg_nofetch()
@@ -406,22 +494,45 @@ pkg_nofetch()
 	done
 }
 
-src_unpack() { 
+src_unpack() {
 	if [ "${A}" != "" ]; then
 		unpack ${A}
 	fi	
 }
 
-src_compile() { 
+src_compile() {
 	if [ -x ./configure ]; then
-		econf 
+		econf
+	fi
+	if [ -f Makefile ] || [ -f GNUmakefile ] || [ -f makefile ]; then
 		emake || die "emake failed"
 	fi
 }
 
-src_install() 
-{ 
-	return 
+src_test()
+{
+	addpredict /
+	if make check -n &> /dev/null; then
+		echo ">>> Test phase [check]: ${CATEGORY}/${PF}"
+		if ! make check; then
+			hasq maketest $FEATURES && die "Make check failed. See above for details."
+			hasq maketest $FEATURES || eerror "Make check failed. See above for details."
+		fi
+	elif make test -n &> /dev/null; then
+		echo ">>> Test phase [test]: ${CATEGORY}/${PF}"
+		if ! make test; then
+			hasq maketest $FEATURES && die "Make test failed. See above for details."
+			hasq maketest $FEATURES || eerror "Make test failed. See above for details."
+		fi
+	else
+		echo ">>> Test phase [none]: ${CATEGORY}/${PF}"
+	fi
+	SANDBOX_PREDICT="${SANDBOX_PREDICT%:/}"
+}
+
+src_install()
+{
+	return
 }
 
 pkg_preinst()
@@ -444,6 +555,11 @@ pkg_postrm()
 	return
 }
 
+pkg_config()
+{
+	eerror "This ebuild does not have a config function."
+}
+
 # Used to generate the /lib/cpp and /usr/bin/cc wrappers
 gen_wrapper() {
 	cat > $1 << END
@@ -523,17 +639,18 @@ dyn_unpack() {
 
 dyn_clean() {
 	rm -rf "${BUILDDIR}/image"
-	rm -rf "${BUILDDIR}/build-info"
 
-	if ! has keeptemp $FEATURES; then
+	if ! hasq keeptemp $FEATURES; then
 		rm -rf "${T}"/*
 	else
 		mv "${T}/environment" "${T}/environment.keeptemp"
 	fi
 
-	if ! has keepwork $FEATURES; then
+	if ! hasq keepwork $FEATURES; then
 		rm -rf "${BUILDDIR}/.compiled"
 		rm -rf "${BUILDDIR}/.unpacked"
+		rm -rf "${BUILDDIR}/.installed"
+		rm -rf "${BUILDDIR}/build-info"
 		rm -rf "${WORKDIR}"
 	fi
 
@@ -591,9 +708,9 @@ insopts() {
 	INSOPTIONS=""
 	for x in $*; do
 		#if we have a debug build, let's not strip anything
-		if has nostrip $FEATURES $RESTRICT && [ "$x" == "-s" ]; then
+		if hasq nostrip $FEATURES $RESTRICT && [ "$x" == "-s" ]; then
 			continue
- 		else
+		else
 			INSOPTIONS="$INSOPTIONS $x"
 		fi
 	done
@@ -612,7 +729,7 @@ exeopts() {
 	EXEOPTIONS=""
 	for x in $*; do
 		#if we have a debug build, let's not strip anything
-		if has nostrip $FEATURES $RESTRICT && [ "$x" == "-s" ]; then
+		if hasq nostrip $FEATURES $RESTRICT && [ "$x" == "-s" ]; then
 			continue
 		else
 			EXEOPTIONS="$EXEOPTIONS $x"
@@ -625,7 +742,7 @@ libopts() {
 	LIBOPTIONS=""
 	for x in $*; do
 		#if we have a debug build, let's not strip anything
-		if has nostrip $FEATURES $RESTRICT && [ "$x" == "-s" ]; then
+		if hasq nostrip $FEATURES $RESTRICT && [ "$x" == "-s" ]; then
 			continue
 		else
 			LIBOPTIONS="$LIBOPTIONS $x"
@@ -641,8 +758,8 @@ abort_handler() {
 	else
 		msg="${EBUILD}: ${1} failed; exiting."
 	fi
-	echo 
-	echo "$msg" 
+	echo
+	echo "$msg"
 	echo
 	eval ${3}
 	#unset signal handler
@@ -669,6 +786,12 @@ abort_package() {
 	exit 1
 }
 
+abort_test() {
+	abort_handler "dyn_test" $1
+	rm -f "${BUILDDIR}/.tested"
+	exit 1
+}
+
 abort_install() {
 	abort_handler "src_install" $1
 	rm -rf "${BUILDDIR}/image"
@@ -684,10 +807,13 @@ dyn_compile() {
 	[ "${LDFLAGS-unset}"     != "unset" ] && export LDFLAGS
 	[ "${ASFLAGS-unset}"     != "unset" ] && export ASFLAGS
 
+	[ "${CCACHE_DIR-unset}"  != "unset" ] && export CCACHE_DIR
+	[ "${CCACHE_SIZE-unset}" != "unset" ] && export CCACHE_SIZE
+
 	[ "${DISTCC_DIR-unset}"  == "unset" ] && export DISTCC_DIR="${PORTAGE_TMPDIR}/.distcc"
 	[ ! -z "${DISTCC_DIR}" ] && addwrite "${DISTCC_DIR}"
 
-	if has noauto $FEATURES &>/dev/null && [ ! -f ${BUILDDIR}/.unpacked ]; then
+	if hasq noauto $FEATURES &>/dev/null && [ ! -f ${BUILDDIR}/.unpacked ]; then
 		echo
 		echo "!!! We apparently haven't unpacked... This is probably not what you"
 		echo "!!! want to be doing... You are using FEATURES=noauto so I'll assume"
@@ -727,32 +853,48 @@ dyn_compile() {
 	#some packages use an alternative to $S to build in, cause
 	#our libtool to create problematic .la files
 	export PWORKDIR="$WORKDIR"
-	src_compile 
-	#|| abort_compile "fail" 
+	src_compile
+	#|| abort_compile "fail"
 	cd "${BUILDDIR}"
 	touch .compiled
 	cd build-info
-	echo "$CBUILD"   > CBUILD
-	echo "$CC"       > CC
-	echo "$CDEPEND"  > CDEPEND
-	echo "$CFLAGS"   > CFLAGS
-	echo "$CHOST"    > CHOST
-	echo "$CXX"      > CXX
-	echo "$CXXFLAGS" > CXXFLAGS
-	echo "$DEPEND"   > DEPEND
-	echo "$IUSE"     > IUSE
-	echo "$PKGUSE"   > PKGUSE
-	echo "$LICENSE"  > LICENSE
-	echo "$CATEGORY" > CATEGORY
-	echo "$PDEPEND"  > PDEPEND
-	echo "$PF"       > PF
-	echo "$PROVIDE"  > PROVIDE
-	echo "$RDEPEND"  > RDEPEND
-	echo "$SLOT"     > SLOT
-	echo "$USE"      > USE
-	set | bzip2 -9 - > environment.bz2
+
+	echo "$ASFLAGS"        > ASFLAGS
+	echo "$CATEGORY"       > CATEGORY
+	echo "$CBUILD"         > CBUILD
+	echo "$CC"             > CC
+	echo "$CDEPEND"        > CDEPEND
+	echo "$CFLAGS"         > CFLAGS
+	echo "$CHOST"          > CHOST
+	echo "$CTARGET"        > CTARGET
+	echo "$CXX"            > CXX
+	echo "$CXXFLAGS"       > CXXFLAGS
+	echo "$DEPEND"         > DEPEND
+	echo "$EXTRA_ECONF"    > EXTRA_ECONF
+	echo "$EXTRA_EINSTALL" > EXTRA_EINSTALL
+	echo "$EXTRA_ECONF"    > EXTRA_EMAKE
+	echo "$FEATURES"       > FEATURES
+	echo "$INHERITED"      > INHERITED
+	echo "$IUSE"           > IUSE
+	echo "$PKGUSE"         > PKGUSE
+	echo "$LDFLAGS"        > LDFLAGS
+	echo "$LIBCFLAGS"      > LIBCFLAGS
+	echo "$LIBCXXFLAGS"    > LIBCXXFLAGS
+	echo "$LICENSE"        > LICENSE
+	echo "$PDEPEND"        > PDEPEND
+	echo "$PF"             > PF
+	echo "$PROVIDE"        > PROVIDE
+	echo "$RDEPEND"        > RDEPEND
+	echo "$RESTRICT"       > RESTRICT
+	echo "$SLOT"           > SLOT
+	echo "$USE"            > USE
+
+	set                                         >  environment
+	export -p | sed 's:declare -rx:declare -x:' >> environment
+	bzip2 -9 environment
+
 	cp "${EBUILD}" "${PF}.ebuild"
-	if has nostrip $FEATURES $RESTRICT; then
+	if hasq nostrip $FEATURES $RESTRICT; then
 		touch DEBUGBUILD
 	fi
 	trap SIGINT SIGQUIT
@@ -777,6 +919,28 @@ dyn_package() {
 	trap SIGINT SIGQUIT
 }
 
+
+dyn_test() {
+	trap "abort_test" SIGINT SIGQUIT
+	if [ -d "${S}" ]; then
+		cd "${S}"
+	fi
+	if hasq maketest $RESTRICT; then
+		ewarn "Skipping make test/check due to ebuild restriction."
+		echo ">>> Test phase [explicitly disabled]: ${CATEGORY}/${PF}"
+	elif ! hasq maketest $FEATURES; then
+		echo ">>> Test phase [not enabled]: ${CATEGORY}/${PF}"
+	else
+		src_test
+	fi
+
+	cd "${BUILDDIR}"
+	touch .tested || die "Failed to 'touch .tested' in ${BUILDDIR}"
+	trap SIGINT SIGQUIT
+}
+	
+
+
 dyn_install() {
 	trap "abort_install" SIGINT SIGQUIT
 	rm -rf "${BUILDDIR}/image"
@@ -792,7 +956,7 @@ dyn_install() {
 	#some packages uses an alternative to $S to build in, cause
 	#our libtool to create problematic .la files
 	export PWORKDIR="$WORKDIR"
-	src_install 
+	src_install
 	#|| abort_install "fail"
 	prepall
 	cd "${D}"
@@ -807,18 +971,63 @@ dyn_install() {
 		echo "UNSAFE SetUID: $i"
 	done
 	
+	if [ -x /usr/bin/readelf -a -x /usr/bin/file ]; then
+		for x in $(find "${D}/" -type f \( -perm -04000 -o -perm -02000 \) ); do
+			f=$(file "${x}")
+			if [ -z "${f/*SB executable*/}" -o -z "${f/*SB shared object*/}" ]; then
+				/usr/bin/readelf -d "${x}" | egrep '\(FLAGS(.*)NOW' > /dev/null
+				if [ $? != 0 ]; then
+					if [ ! -z "${f/*statically linked*/}" ]; then
+						#uncomment this line out after developers have had ample time to fix pkgs.
+						#UNSAFE=$(($UNSAFE + 1))
+						echo -ne '\a'
+						echo "QA Notice: ${x:${#D}:${#x}} is setXid, dynamically linked and using lazy bindings."
+						echo "This combination is generally discouraged. Try: LDFLAGS='-Wl,-z,now' emerge ${PN}"
+						echo -ne '\a'
+						sleep 1
+					fi
+				fi
+			fi
+		done
+	fi
+
 	if [[ $UNSAFE > 0 ]]; then
 		die "There are ${UNSAFE} unsafe files. Portage will not install them."
 	fi
 	
-	find "${D}/" -user  portage -print0 | $XARGS -0 -n100 chown root
-	if [ "$USERLAND" == "BSD" ]; then
-		find "${D}/" -group portage -print0 | $XARGS -0 -n100 chgrp wheel
-	else
-		find "${D}/" -group portage -print0 | $XARGS -0 -n100 chgrp root
+	function stat_perms() {
+		local f
+		f=$(stat -c '%f' "$1")
+		f=$(printf %o 0x$f)
+		f="${f:${#f}-4}"
+		echo $f
+	}
+
+	local file s
+	find "${D}/" -user  portage | while read file; do
+		ewarn "file $file was installed with user portage!"
+		s=$(stat_perms $file)
+		chown root "$file"
+		chmod "$s" "$file"
+	done
+
+	find "${D}/" -group portage | while read file; do
+		ewarn "file $file was installed with group portage!"
+		s=$(stat_perms "$file")
+		[ "$USERLAND" == "BSD" ] && chgrp wheel "$file"
+		[ "$USERLAND" != "BSD" ] && chgrp root "$file"
+		chmod "$s" "$file"
+	done
+
+	unset -f stat_perms
+
+	# Portage regenerates this on the installed system.
+	if [ -f "${D}/usr/share/info/dir.gz" ]; then
+		rm -f "${D}/usr/share/info/dir.gz"
 	fi
 
-	echo ">>> Completed installing into ${D}"
+	touch "${BUILDDIR}/.installed"
+	echo ">>> Completed installing ${PF} into ${D}"
 	echo
 	cd ${BUILDDIR}
 	trap SIGINT SIGQUIT
@@ -832,22 +1041,27 @@ dyn_preinst() {
 	pkg_preinst
 
 	# remove man pages
-	if has noman $FEATURES; then
+	if hasq noman $FEATURES; then
 		rm -fR "${IMAGE}/usr/share/man"
 	fi
 
 	# remove info pages
-	if has noinfo $FEATURES; then
+	if hasq noinfo $FEATURES; then
 		rm -fR "${IMAGE}/usr/share/info"
 	fi
 
 	# remove docs
-	if has nodoc $FEATURES; then
+	if hasq nodoc $FEATURES; then
 		rm -fR "${IMAGE}/usr/share/doc"
 	fi
 
+	# remove share dir if unnessesary
+	if hasq nodoc $FEATURES -o hasq noman $FEATURES -o hasq noinfo $FEATURES; then
+		rmdir "${IMAGE}/usr/share" &> /dev/null
+	fi
+
 	# Smart FileSystem Permissions
-	if has sfperms $FEATURES; then
+	if hasq sfperms $FEATURES; then
 		for i in $(find ${IMAGE}/ -type f -perm -4000); do
 			ebegin ">>> SetUID: [chmod go-r] $i "
 			chmod go-r "$i"
@@ -861,7 +1075,7 @@ dyn_preinst() {
 	fi
 
 	# total suid control.
-	if has suidctl $FEATURES > /dev/null ; then
+	if hasq suidctl $FEATURES > /dev/null ; then
 		sfconf=/etc/portage/suidctl.conf
 		echo ">>> Preforming suid scan in ${IMAGE}"
 		for i in $(find ${IMAGE}/ -type f \( -perm -4000 -o -perm -2000 \) ); do
@@ -870,8 +1084,8 @@ dyn_preinst() {
 				if [ "${suid}" = "${i/${IMAGE}/}" ]; then
 					echo "- ${i/${IMAGE}/} is an approved suid file"
 				else
-					for x in 5 4 3 2 1 0; do echo -ne "\a"; sleep 0.25 ; done
 					echo ">>> Removing sbit on non registered ${i/${IMAGE}/}"
+					for x in 5 4 3 2 1 0; do echo -ne "\a"; sleep 0.25 ; done
 					echo -ne "\a"
 					chmod ugo-s "${i}"
 					grep ^#${i/${IMAGE}/}$ ${sfconf} > /dev/null || {
@@ -880,7 +1094,8 @@ dyn_preinst() {
 						# can easly be bypassed using the addwrite() function
 						addwrite "${sfconf}"
 						echo ">>> Appending commented out entry to ${sfconf} for ${PF}"
-						ls -ldh "${i}" | awk '{print "## "$0}' | sed s:"${IMAGE}"::g >> ${sfconf}
+						ls_ret=`ls -ldh "${i}"`
+						echo "## ${ls_ret%${IMAGE}*}${ls_ret#*${IMAGE}}" >> ${sfconf}
 						echo "#${i/${IMAGE}/}" >> ${sfconf}
 						# no delwrite() eh?
 						# delwrite ${sconf}
@@ -893,7 +1108,7 @@ dyn_preinst() {
 	fi
 
 	# SELinux file labeling (needs to always be last in dyn_preinst)
-	if use selinux; then
+	if useq selinux; then
 		# only attempt to label if setfiles is executable
 		# and 'context' is available on selinuxfs.
 		if [ -f /selinux/context -a -x /usr/sbin/setfiles ]; then
@@ -950,54 +1165,64 @@ __END1__
 
 dyn_rpm() {
 	dyn_spec
-	rpm -bb "${PF}.spec" || die "Failed to integrate rpm spec file"
+	rpmbuild -bb "${PF}.spec" || die "Failed to integrate rpm spec file"
 	install -D "/usr/src/redhat/RPMS/i386/${PN}-${PV}-${PR}.i386.rpm" "${RPMDIR}/${CATEGORY}/${PN}-${PV}-${PR}.rpm" || die "Failed to move rpm"
 }
 
 dyn_help() {
 	echo
 	echo "Portage"
-	echo "Copyright 2002 Gentoo Technologies, Inc."
-	echo 
+	echo "Copyright 1999-2004 Gentoo Foundation"
+	echo
 	echo "How to use the ebuild command:"
-	echo 
+	echo
 	echo "The first argument to ebuild should be an existing .ebuild file."
 	echo
 	echo "One or more of the following options can then be specified.  If more"
 	echo "than one option is specified, each will be executed in order."
 	echo
+	echo "  help        : show this help screen"
 	echo "  setup       : execute package specific setup actions"
 	echo "  fetch       : download source archive(s) and patches"
+	echo "  digest      : creates a digest and a manifest file for the package"
+	echo "  manifest    : creates a manifest file for the package"
 	echo "  unpack      : unpack/patch sources (auto-fetch if needed)"
 	echo "  compile     : compile sources (auto-fetch/unpack if needed)"
-	echo "  merge       : merge image into live filesystem, recording files in db"
-	echo "                (auto-fetch/unpack/compile if needed)"
+	echo "  test        : test package (auto-fetch/unpack/compile if needed)"
+	echo "  preinst     : execute pre-install instructions"
+	echo "  postinst    : execute post-install instructions"
+	echo "  install     : installs the package to the temporary install directory"
+	echo "  qmerge      : merge image into live filesystem, recording files in db"
+	echo "  merge       : does fetch, unpack, compile, install and qmerge"
+	echo "  prerm       : execute pre-removal instructions"
+	echo "  postrm      : execute post-removal instructions"
 	echo "  unmerge     : remove package from live filesystem"
-	echo "  package     : create tarball package of type ${PACKAGE}"
-	echo "                (will be stored in ${PKGDIR}/All)"
+	echo "  config      : execute package specific configuration actions"
+	echo "  package     : create tarball package in ${PKGDIR}/All"
+	echo "  rpm         : builds a RedHat RPM package"
 	echo "  clean       : clean up all source and temporary files"
 	echo
 	echo "The following settings will be used for the ebuild process:"
 	echo
-	echo "  package     : ${PF}" 
-	echo "  slot        : ${SLOT}" 
-	echo "  category    : ${CATEGORY}" 
+	echo "  package     : ${PF}"
+	echo "  slot        : ${SLOT}"
+	echo "  category    : ${CATEGORY}"
 	echo "  description : ${DESCRIPTION}"
-	echo "  system      : ${CHOST}" 
-	echo "  c flags     : ${CFLAGS}" 
-	echo "  c++ flags   : ${CXXFLAGS}" 
-	echo "  make flags  : ${MAKEOPTS}" 
+	echo "  system      : ${CHOST}"
+	echo "  c flags     : ${CFLAGS}"
+	echo "  c++ flags   : ${CXXFLAGS}"
+	echo "  make flags  : ${MAKEOPTS}"
 	echo -n "  build mode  : "
-	if has nostrip $FEATURES $RESTRICT;	then
+	if hasq nostrip $FEATURES $RESTRICT; then
 		echo "debug (large)"
 	else
 		echo "production (stripped)"
 	fi
-	echo "  merge to    : ${ROOT}" 
+	echo "  merge to    : ${ROOT}"
 	echo
 	if [ -n "$USE" ]; then
 		echo "Additionally, support for the following optional features will be enabled:"
-		echo 
+		echo
 		echo "  ${USE}"
 	fi
 	echo
@@ -1013,7 +1238,7 @@ dyn_help() {
 
 # in the future might use e* from /etc/init.d/functions.sh if i feel like it
 debug-print() {
-	# if $T isn't defined, we're in dep calculation mode and 
+	# if $T isn't defined, we're in dep calculation mode and
 	# shouldn't do anything
 	[ -z "$T" ] && return 0
 
@@ -1038,7 +1263,7 @@ debug-print() {
 # The following 2 functions are debug-print() wrappers
 
 debug-print-function() {
-	str="$1: entering function" 
+	str="$1: entering function"
 	shift
 	debug-print "$str, parameters: $*"
 }
@@ -1063,7 +1288,6 @@ inherit() {
 	local B_RDEPEND
 	local B_CDEPEND
 	local B_PDEPEND
-
 	while [ "$1" ]; do
 		location="${ECLASSDIR}/${1}.eclass"
 
@@ -1071,6 +1295,14 @@ inherit() {
 		PECLASS="$ECLASS"
 		export ECLASS="$1"
 
+		if [ "$EBUILD_PHASE" != "depend" ]; then
+			if ! hasq $ECLASS $INHERITED; then
+				echo
+				echo "QA Notice: ECLASS '$ECLASS' inherited illegally in $CATEGORY/$PF" >&2
+				echo
+			fi
+		fi
+
 		# any future resolution code goes here
 		if [ -n "$PORTDIR_OVERLAY" ]; then
 			local overlay
@@ -1079,7 +1311,6 @@ inherit() {
 				if [ -e "$olocation" ]; then
 					location="${olocation}"
 					debug-print "  eclass exists: ${location}"
-					break
 				fi
 			done
 		fi
@@ -1092,12 +1323,13 @@ inherit() {
 		set -f
 
 		# Retain the old data and restore it later.
-		unset B_DEPEND B_RDEPEND B_CDEPEND B_PDEPEND
+		unset B_IUSE B_DEPEND B_RDEPEND B_CDEPEND B_PDEPEND
+		[ "${IUSE-unset}"    != "unset" ] && B_IUSE="${IUSE}"
 		[ "${DEPEND-unset}"  != "unset" ] && B_DEPEND="${DEPEND}"
 		[ "${RDEPEND-unset}" != "unset" ] && B_RDEPEND="${RDEPEND}"
 		[ "${CDEPEND-unset}" != "unset" ] && B_CDEPEND="${CDEPEND}"
 		[ "${PDEPEND-unset}" != "unset" ] && B_PDEPEND="${PDEPEND}"
-		unset   DEPEND   RDEPEND   CDEPEND   PDEPEND
+		unset   IUSE   DEPEND   RDEPEND   CDEPEND   PDEPEND
 		#turn on glob expansion
 		set +f
 		
@@ -1109,11 +1341,15 @@ inherit() {
 
 		# If each var has a value, append it to the global variable E_* to
 		# be applied after everything is finished. New incremental behavior.
+		[ "${IUSE-unset}"    != "unset" ] && export E_IUSE="${E_IUSE} ${IUSE}"
 		[ "${DEPEND-unset}"  != "unset" ] && export E_DEPEND="${E_DEPEND} ${DEPEND}"
 		[ "${RDEPEND-unset}" != "unset" ] && export E_RDEPEND="${E_RDEPEND} ${RDEPEND}"
 		[ "${CDEPEND-unset}" != "unset" ] && export E_CDEPEND="${E_CDEPEND} ${CDEPEND}"
 		[ "${PDEPEND-unset}" != "unset" ] && export E_PDEPEND="${E_PDEPEND} ${PDEPEND}"
 
+		[ "${B_IUSE-unset}"    != "unset" ] && IUSE="${B_IUSE}"
+		[ "${B_IUSE-unset}"    != "unset" ] || unset IUSE
+
 		[ "${B_DEPEND-unset}"  != "unset" ] && DEPEND="${B_DEPEND}"
 		[ "${B_DEPEND-unset}"  != "unset" ] || unset DEPEND
 
@@ -1127,7 +1363,7 @@ inherit() {
 		[ "${B_PDEPEND-unset}" != "unset" ] || unset PDEPEND
 
 		#turn on glob expansion
- 		set +f
+		set +f
 		
 		hasq $1 $INHERITED || export INHERITED="$INHERITED $1"
 
@@ -1148,7 +1384,7 @@ EXPORT_FUNCTIONS() {
 		exit 1
 	fi
 	while [ "$1" ]; do
-		debug-print "EXPORT_FUNCTIONS: ${1} -> ${ECLASS}_${1}" 
+		debug-print "EXPORT_FUNCTIONS: ${1} -> ${ECLASS}_${1}"
 		eval "$1() { ${ECLASS}_$1 "\$@" ; }" > /dev/null
 		shift
 	done
@@ -1174,8 +1410,8 @@ newdepend() {
 			do_newdepend DEPEND sys-devel/autoconf sys-devel/automake sys-devel/make
 			;;
 		"/c")
-			do_newdepend DEPEND sys-devel/gcc virtual/glibc
-			do_newdepend RDEPEND virtual/glibc
+			do_newdepend DEPEND sys-devel/gcc virtual/libc
+			do_newdepend RDEPEND virtual/libc
 			;;
 		*)
 			do_newdepend DEPEND $1
@@ -1216,11 +1452,31 @@ do_newdepend() {
 		# variable. This is a sneaky way to make this infinately expandable.
 		# The normal translation of this would look something like this:
 		# E_DEPEND="${E_DEPEND} $1"  ::::::  Cool, huh? :)
-		eval export ${TARGET}=\"\${${TARGET}} $1\"
+		eval export ${TARGET}=\"\${${TARGET}} \$1\"
 		shift
 	done
 }
 
+# this is a function for removing any directory matching a passed in pattern from
+# PATH
+remove_path_entry() {
+	save_IFS
+	IFS=":"
+	stripped_path="${PATH}"
+	while [ -n "$1" ]; do
+		cur_path=""
+		for p in ${stripped_path}; do
+			if [ "${p/${1}}" == "${p}" ]; then
+				cur_path="${cur_path}:${p}"
+			fi
+		done
+		stripped_path="${cur_path#:*}"
+		shift
+	done
+	restore_IFS
+	PATH="${stripped_path}"
+}
+
 # === === === === === === === === === === === === === === === === === ===
 # === === === === === functions end, main part begins === === === === ===
 # === === === === === functions end, main part begins === === === === ===
@@ -1235,12 +1491,12 @@ if [ "$*" != "depend" ] && [ "$*" != "clean" ]; then
 		export USER=portage
 	fi
 
-	if has distcc ${FEATURES} &>/dev/null; then
+	if hasq distcc ${FEATURES} &>/dev/null; then
 		if [ -d /usr/lib/distcc/bin ]; then
 			#We can enable distributed compile support
 			if [ -z "${PATH/*distcc*/}" ]; then
 				# Remove the other reference.
-				PATH="$(echo ${PATH} | sed 's/:[^:]*distcc[^:]*:/:/;s/^[^:]*distcc[^:]*://;s/:[^:]*distcc[^:]*$//')"
+				remove_path_entry "distcc"
 			fi
 			export PATH="/usr/lib/distcc/bin:${PATH}"
 			[ ! -z "${DISTCC_LOG}" ] && addwrite "$(dirname ${DISTCC_LOG})"
@@ -1250,11 +1506,11 @@ if [ "$*" != "depend" ] && [ "$*" != "clean" ]; then
 		fi
 	fi
 
-	if has ccache ${FEATURES} &>/dev/null; then
+	if hasq ccache ${FEATURES} &>/dev/null; then
 		#We can enable compiler cache support
 		if [ -z "${PATH/*ccache*/}" ]; then
 			# Remove the other reference.
-			PATH="$(echo ${PATH} | sed 's/:[^:]*ccache[^:]*:/:/;s/^[^:]*ccache[^:]*://;s/:[^:]*ccache[^:]*$//')"
+			remove_path_entry "ccache"
 		fi
 
 		if [ -d /usr/lib/ccache/bin ]; then
@@ -1268,7 +1524,7 @@ if [ "$*" != "depend" ] && [ "$*" != "clean" ]; then
 		addread "${CCACHE_DIR}"
 		addwrite "${CCACHE_DIR}"
 
-		[ -z "${CCACHE_SIZE}" ] && export CCACHE_SIZE="2G"
+		[ -z "${CCACHE_SIZE}" ] && export CCACHE_SIZE="500M"
 		ccache -M ${CCACHE_SIZE} &> /dev/null
 	fi
 
@@ -1281,7 +1537,7 @@ else
 
 killparent() {
 	trap INT
-	kill -KILL ${PORTAGE_MASTER_PID}
+	kill ${PORTAGE_MASTER_PID}
 }
 trap "killparent" INT
 
@@ -1290,18 +1546,64 @@ fi # "$*"!="depend" && "$*"!="clean"
 export SANDBOX_ON="1"
 export S=${WORKDIR}/${P}
 
-unset   DEPEND   RDEPEND   CDEPEND   PDEPEND
-unset E_DEPEND E_RDEPEND E_CDEPEND E_PDEPEND
+unset E_IUSE E_DEPEND E_RDEPEND E_CDEPEND E_PDEPEND
 
-export EBUILD_PHASE="$*"
+declare -r T P PN PV PVR PR A D EBUILD EMERGE_FROM O PPID FILESDIR
+declare -r PORTAGE_TMPDIR
+
+# Turn of extended glob matching so that g++ doesn't get incorrectly matched.
+shopt -u extglob
+
+QA_INTERCEPTORS="javac java-config python python-config perl grep egrep fgrep sed gcc g++ cc bash awk nawk gawk pkg-config"
+# level the QA interceptors if we're in depend
+if hasq "depend" "$@"; then
+	for BIN in ${QA_INTERCEPTORS}; do
+		BIN_PATH=`type -pf ${BIN}`
+		if [ "$?" != "0" ]; then
+			BODY="echo \"*** missing command: ${BIN}\" >&2; return 127"
+		else
+			BODY="${BIN_PATH} \"\$@\"; return \$?"
+		fi
+		FUNC_SRC="${BIN}() {
+		echo -n \"QA Notice: ${BIN} in global scope: \" >&2
+		if [ \$ECLASS_DEPTH -gt 0 ]; then
+			echo \"eclass \${ECLASS}\" >&2
+		else
+			echo \"\${CATEGORY}/\${PF}\" >&2
+		fi
+		${BODY}
+		}";
+		eval "$FUNC_SRC" || echo "error creating QA interceptor ${BIN}" >&2
+	done
+	unset src bin_path body
+fi
 source ${EBUILD} || die "error sourcing ebuild"
+if ! hasq depend $EBUILD_PHASE; then
+	RESTRICT="${PORTAGE_RESTRICT}"
+	unset PORTAGE_RESTRICT
+fi
 [ -z "${ERRORMSG}" ] || die "${ERRORMSG}"
 
+# Expand KEYWORDS
+# We need to turn off pathname expansion for -* in KEYWORDS and
+# we need to escape ~ to avoid tilde expansion
+set -f
+KEYWORDS="`eval echo ${KEYWORDS//~/\\~}`"
+set +f
+
+hasq nostrip ${RESTRICT} && export DEBUGBUILD=1
+
 #a reasonable default for $S
 if [ "$S" = "" ]; then
 	export S=${WORKDIR}/${P}
 fi
 
+#wipe the interceptors.  we don't want saved.
+if hasq "depend" "$@"; then
+	unset -f $QA_INTERCEPTORS
+	unset QA_INTERCEPTORS
+fi
+
 #some users have $TMP/$TMPDIR to a custom dir in their home ...
 #this will cause sandbox errors with some ./configure
 #scripts, so set it to $T.
@@ -1314,18 +1616,26 @@ export TMPDIR="${T}"
 #turn off glob expansion from here on in to prevent *'s and ? in the DEPEND
 #syntax from getting expanded :)  Fixes bug #1473
 set -f
-#if [ -z "`set | grep ^RDEPEND=`" ]; then
 if [ "${RDEPEND-unset}" == "unset" ]; then
 	export RDEPEND=${DEPEND}
 	debug-print "RDEPEND: not set... Setting to: ${DEPEND}"
 fi
 
 #add in dependency info from eclasses
+IUSE="$IUSE $E_IUSE"
 DEPEND="$DEPEND $E_DEPEND"
 RDEPEND="$RDEPEND $E_RDEPEND"
 CDEPEND="$CDEPEND $E_CDEPEND"
 PDEPEND="$PDEPEND $E_PDEPEND"
 
+unset E_IUSE E_DEPEND E_RDEPEND E_CDEPEND E_PDEPEND
+
+if [ "${EBUILD_PHASE}" != "depend" ]; then
+	# Lock the dbkey variables after the global phase
+	declare -r DEPEND RDEPEND SLOT SRC_URI RESTRICT HOMEPAGE LICENSE DESCRIPTION
+	declare -r KEYWORDS INHERITED IUSE CDEPEND PDEPEND PROVIDE
+fi
+
 set +f
 
 for myarg in $*; do
@@ -1346,7 +1656,7 @@ for myarg in $*; do
 			set +x
 		fi
 		;;
-	unpack|compile|clean|install)
+	unpack|compile|test|clean|install)
 		if [ "${SANDBOX_DISABLED="0"}" == "0" ]; then
 			export SANDBOX_ON="1"
 		else
@@ -1414,6 +1724,15 @@ for myarg in $*; do
 		echo `echo "$IUSE"`        >> $dbkey
 		echo `echo "$CDEPEND"`     >> $dbkey
 		echo `echo "$PDEPEND"`     >> $dbkey
+		echo `echo "$PROVIDE"`     >> $dbkey
+		echo `echo "$UNUSED_01"`   >> $dbkey
+		echo `echo "$UNUSED_02"`   >> $dbkey
+		echo `echo "$UNUSED_03"`   >> $dbkey
+		echo `echo "$UNUSED_04"`   >> $dbkey
+		echo `echo "$UNUSED_05"`   >> $dbkey
+		echo `echo "$UNUSED_06"`   >> $dbkey
+		echo `echo "$UNUSED_07"`   >> $dbkey
+		echo `echo "$UNUSED_08"`   >> $dbkey
 		set +f
 		#make sure it is writable by our group:
 		exit 0
@@ -1439,8 +1758,5 @@ if [ "$myarg" != "clean" ]; then
 	chown portage:portage "${T}/environment" &>/dev/null
 	chmod g+w "${T}/environment" &>/dev/null
 fi
-touch "${T}/successful"  &>/dev/null
-chown portage:portage "${T}/successful" &>/dev/null
-chmod g+w "${T}/successful" &>/dev/null
 
 exit 0
