@@ -1,5 +1,7 @@
 #!/usr/bin/perl -w
-# $Header: /home/cvsroot/gentoo-src/portage/bin/g-cpan.pl,v 1.12 2004/01/22 05:51:45 carpaski Exp $
+# Copyright 1999-2004 Gentoo Foundation
+# Distributed under the terms of the GNU General Public License v2
+# $Header: /var/cvsroot/gentoo-src/portage/bin/g-cpan.pl,v 1.16 2004/10/20 17:40:54 carpaski Exp $
 
 # History: 
 
@@ -83,6 +85,9 @@ my $TMP_DEV_PERL_DIR = '/var/tmp/db/dev-perl';
 my $MAKECONF         = '/etc/make.conf';
 my ( $OVERLAY_DIR, $PORTAGE_DIR, $PORTAGE_DEV_PERL, $PORTAGE_DISTDIR ) = get_globals();
 
+# Create the ebuild in PORTDIR_OVERLAY, if it is defined and exists
+$tmp_overlay_dir = $OVERLAY_DIR unless $OVERLAY_DIR eq "";
+
 my $arches = join( ' ', map { chomp; $_ } `cat $PORTAGE_DIR/profiles/arch.list` );
 
 #this should never find the dir, but just to be safe
@@ -121,10 +126,14 @@ sub ebuild_exists {
     # check the main portage tree
     return 1
       if ( ( -d File::Spec->catfile( $PORTAGE_DEV_PERL, $dir ) )
-        || ( -d File::Spec->catfile( $OVERLAY_DIR,      $dir ) )
-        || ( -d File::Spec->catfile( $tmp_overlay_dir,  $dir ) )
+        || ( -d File::Spec->catfile( $perldev_overlay,  $dir ) )
         || ( -d File::Spec->catfile( $TMP_DEV_PERL_DIR, $dir ) ) );
 
+    # check for ebuilds that have be created by g-cpan.pl
+    for my $ebuild ( @ebuild_list ) {
+        return 1 if ( $ebuild eq $dir );
+    }
+
     return 0;
 }
 
@@ -178,7 +187,7 @@ sub create_ebuild {
     print EBUILD <<"HERE";
 
 
-# Copyright 1999-2003 Gentoo Technologies, Inc.
+# Copyright 1999-2004 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
 
 inherit perl-module
@@ -276,10 +285,10 @@ sub install_module {
     if ($have_digestmd5) {
     open( DIGIFILE, $localfile ) or die "Can't open '$file': $!";
     binmode(DIGIFILE);
-	    $md5digest = Digest::MD5->new->addfile(*DIGIFILE)->hexdigest;
+    $md5digest = Digest::MD5->new->addfile(*DIGIFILE)->hexdigest;
     close(DIGIFILE);
     } else {
-    	    ($md5digest = qx(/usr/bin/md5sum $localfile)) =~ s/^(.*?)\s.*$/$1/s;
+        ($md5digest = qx(/usr/bin/md5sum $localfile)) =~ s/^(.*?)\s.*$/$1/s;
     }
 
     my $md5string = sprintf "MD5 %s %s %d", $md5digest, $base,
@@ -307,14 +316,15 @@ sub clean_up {
     $ENV{'PORTDIR_OVERLAY'} = $OVERLAY_DIR;
 
     #Clean out the /tmp tree we were using
-    rmtree( ["$tmp_overlay_dir"] );
+    rmtree( ["$tmp_overlay_dir"] ) if $OVERLAY_DIR eq "";
 }
 
 sub emerge_module {
     foreach my $ebuild_name (@ebuild_list) {
         $ebuild_name =~ m/.*\/(.*)-[^-]+\./;
         print "emerging $ebuild_name\n";
-        system("emerge $ebuild_name");
+#       system("emerge $ebuild_name");
+	system( "emerge", "--digest", $ebuild_name );
 
     }
 }
