@@ -1,7 +1,7 @@
 # cvstree.py -- cvs tree utilities
-# Copyright 1998-2003 Gentoo Technologies, Inc.
-# Distributed under the GNU Public License v2
-# $Header: /home/cvsroot/gentoo-src/portage/pym/cvstree.py,v 1.6 2003/11/10 08:30:55 carpaski Exp $
+# Copyright 1998-2004 Gentoo Foundation
+# Distributed under the terms of the GNU General Public License v2
+# $Header: /var/cvsroot/gentoo-src/portage/pym/cvstree.py,v 1.12 2004/10/11 04:12:02 carpaski Exp $
 
 import string,os,time,sys,re
 from stat import *
@@ -117,6 +117,8 @@ def findunadded(entries,recursive=0,basedir=""):
 	if basedir and basedir[-1]!="/":
 		basedir=basedir+"/"
 	mylist=[]
+
+	#ignore what cvs ignores.
 	for myfile in entries["files"].keys():
 		if "cvs" not in entries["files"][myfile]["status"]:
 			mylist.append(basedir+myfile)
@@ -154,6 +156,16 @@ def findall(entries, recursive=0, basedir=""):
 	myremoved = findremoved(entries,recursive,basedir)
 	return [mynew, mychanged, mymissing, myunadded, myremoved]
 
+ignore_list = re.compile("(^|/)(RCS(|LOG)|SCCS|CVS(|\.adm)|cvslog\..*|tags|TAGS|\.(make\.state|nse_depinfo)|.*~|(\.|)#.*|,.*|_$.*|.*\$|\.del-.*|.*\.(old|BAK|bak|orig|rej|a|olb|o|obj|so|exe|Z|elc|ln)|core)$")
+def apply_cvsignore_filter(list):
+	x=0
+	while x < len(list):
+		if ignore_list.match(list[x].split("/")[-1]):
+			list.pop(x)
+		else:
+			x+=1
+	return list
+	
 def getentries(mydir,recursive=0):
 	"""(basedir,recursive=0)
 	Scans the given directory and returns an datadict of all the entries in
@@ -167,6 +179,8 @@ def getentries(mydir,recursive=0):
 		myfile=open(myfn, "r")
 		mylines=myfile.readlines()
 		myfile.close()
+	except SystemExit, e:
+		raise
 	except:
 		mylines=[]
 	for line in mylines:
@@ -203,7 +217,8 @@ def getentries(mydir,recursive=0):
 			entries["files"][mysplit[1]]["status"]=["cvs"]
 			if entries["files"][mysplit[1]]["revision"][0]=="-":
 				entries["files"][mysplit[1]]["status"]+=["removed"]
-	for file in os.listdir(mydir):
+
+	for file in apply_cvsignore_filter(os.listdir(mydir)):
 		if file=="CVS":
 			continue
 		if file=="digest-framerd-2.4.3":
@@ -250,6 +265,8 @@ def getentries(mydir,recursive=0):
 					print "stat done"
 				
 				del mystat
+			except SystemExit, e:
+				raise
 			except Exception, e:
 				print "failed to stat",file
 				print e
