@@ -1,18 +1,22 @@
 # getbinpkg.py -- Portage binary-package helper functions
-# Copyright 2003 Gentoo Technologies, Inc.
-# Distributed under the GNU Public License v2
-# $Id: getbinpkg.py,v 1.5 2004/02/04 22:40:21 carpaski Exp $
+# Copyright 2003-2004 Gentoo Foundation
+# Distributed under the terms of the GNU General Public License v2
+# $Header: /var/cvsroot/gentoo-src/portage/pym/getbinpkg.py,v 1.12.2.2 2004/10/29 14:51:09 jstubbs Exp $
 
 from output import *
 import htmllib,HTMLParser,string,formatter,sys,os,xpak,time,tempfile,cPickle,base64
 
 try:
 	import ftplib
+except SystemExit, e:
+	raise
 except Exception, e:
 	sys.stderr.write(red("!!! CANNOT IMPORT FTPLIB: ")+str(e)+"\n")
 
 try:
 	import httplib
+except SystemExit, e:
+	raise
 except Exception, e:
 	sys.stderr.write(red("!!! CANNOT IMPORT HTTPLIB: ")+str(e)+"\n")
 
@@ -112,13 +116,17 @@ def create_conn(baseurl,conn=None):
 		elif protocol == "http":
 			conn = httplib.HTTPConnection(host)
 		elif protocol == "ftp":
+			passive = 1
+			if(host[-1] == "*"):
+				passive = 0
+				host = host[:-1]
 			conn = ftplib.FTP(host)
 			if password:
 				conn.login(username,password)
 			else:
 				sys.stderr.write(yellow(" * No password provided for username")+" '"+str(username)+"'\n\n")
 				conn.login(username)
-			conn.set_pasv(1)
+			conn.set_pasv(passive)
 			conn.set_debuglevel(0)
 		else:
 			raise NotImplementedError, "%s is not a supported protocol." % protocol
@@ -177,13 +185,17 @@ def make_http_request(conn, address, params={}, headers={}, dest=None):
 	the data from address, performing Location forwarding and using the
 	optional params and headers."""
 
-	rc = 301
+	rc = 0
 	response = None
-	while ((rc == 301) or (rc == 302)):
+	while (rc == 0) or (rc == 301) or (rc == 302):
 		try:
+			if (rc != 0):
+				conn,ignore,ignore,ignore,ignore = create_conn(address)
 			conn.request("GET", address, params, headers)
+		except SystemExit, e:
+			raise
 		except Exception, e:
-			return None,None,"Server request failed:",e[1]
+			return None,None,"Server request failed: "+str(e)
 		response = conn.getresponse()
 		rc = response.status
 
@@ -272,7 +284,7 @@ def dir_get_list(baseurl,conn=None):
 			del page
 			listing = parser.get_anchors()
 		else:
-			raise Exception, "Unable to get listing: %s" % (rc,msg)
+			raise Exception, "Unable to get listing: %s %s" % (rc,msg)
 	elif protocol in ["ftp"]:
 		if address[-1] == '/':
 			olddir = conn.pwd()
@@ -343,6 +355,7 @@ def file_get(baseurl,dest,conn=None,fcmd=None):
 
 	fcmd = string.replace(fcmd, "${DISTDIR}", dest)
 	fcmd = string.replace(fcmd, "${URI}", baseurl)
+	fcmd = string.replace(fcmd, "${FILE}", os.path.basename(baseurl))
 	mysplit = string.split(fcmd)
 	mycmd   = mysplit[0]
 	myargs  = [os.path.basename(mycmd)]+mysplit[1:]
@@ -388,7 +401,7 @@ def file_get_lib(baseurl,dest,conn=None):
 	return rc
 
 
-def dir_get_metadata(baseurl,conn=None, chunk_size=3000, verbose=1, usingcache=1, makepickle=None):
+def dir_get_metadata(baseurl, conn=None, chunk_size=3000, verbose=1, usingcache=1, makepickle=None):
 	"""(baseurl,conn,chunk_size,verbose) -- 
 	"""
 	if not conn:
@@ -408,6 +421,8 @@ def dir_get_metadata(baseurl,conn=None, chunk_size=3000, verbose=1, usingcache=1
 		metadata = cPickle.load(metadatafile)
 		sys.stderr.write("Loaded metadata pickle.\n")
 		metadatafile.close()
+	except SystemExit, e:
+		raise
 	except:
 		metadata = {}
 	if not metadata.has_key(baseurl):
@@ -456,6 +471,8 @@ def dir_get_metadata(baseurl,conn=None, chunk_size=3000, verbose=1, usingcache=1
 						mytempfile.seek(0)
 						gzindex = gzip.GzipFile(mfile[:-3],'rb',9,mytempfile)
 						data = gzindex.read()
+					except SystemExit, e:
+						raise
 					except Exception, e:
 						mytempfile.close()
 						sys.stderr.write("!!! Failed to use gzip: "+str(e)+"\n")
@@ -468,6 +485,8 @@ def dir_get_metadata(baseurl,conn=None, chunk_size=3000, verbose=1, usingcache=1
 					metadata[baseurl]["modified"]  = 0 # It's not, right after download.
 					sys.stderr.write("Pickle loaded.\n")
 					break
+				except SystemExit, e:
+					raise
 				except Exception, e:
 					sys.stderr.write("!!! Failed to read data from index: "+str(mfile)+"\n")
 					sys.stderr.write("!!! "+str(e)+"\n")
@@ -475,6 +494,8 @@ def dir_get_metadata(baseurl,conn=None, chunk_size=3000, verbose=1, usingcache=1
 				metadatafile = open("/var/cache/edb/remote_metadata.pickle", "w+")
 				cPickle.dump(metadata,metadatafile)
 				metadatafile.close()
+			except SystemExit, e:
+				raise
 			except Exception, e:
 				sys.stderr.write("!!! Failed to write binary metadata to disk!\n")
 				sys.stderr.write("!!! "+str(e)+"\n")
@@ -498,7 +519,7 @@ def dir_get_metadata(baseurl,conn=None, chunk_size=3000, verbose=1, usingcache=1
 	sys.stderr.write("\n")
 	
 	try:
-		if metadata[baseurl]["modified"]:
+		if metadata[baseurl].has_key("modified") and metadata[baseurl]["modified"]:
 			metadata[baseurl]["timestamp"] = int(time.time())
 			metadatafile = open("/var/cache/edb/remote_metadata.pickle", "w+")
 			cPickle.dump(metadata,metadatafile)
@@ -507,6 +528,8 @@ def dir_get_metadata(baseurl,conn=None, chunk_size=3000, verbose=1, usingcache=1
 			metadatafile = open(makepickle, "w")
 			cPickle.dump(metadata[baseurl]["data"],metadatafile)
 			metadatafile.close()
+	except SystemExit, e:
+		raise
 	except Exception, e:
 		sys.stderr.write("!!! Failed to write binary metadata to disk!\n")
 		sys.stderr.write("!!! "+str(e)+"\n")
