@@ -1,7 +1,7 @@
-#
-#
-#
-# $Header: /home/cvsroot/gentoo-src/portage/pym/xpak.py,v 1.11 2003/12/25 03:06:09 carpaski Exp $
+# Copyright 2001-2004 Gentoo Foundation
+# Distributed under the terms of the GNU General Public License v2
+# $Header: /var/cvsroot/gentoo-src/portage/pym/xpak.py,v 1.13.2.2 2005/01/16 02:35:33 carpaski Exp $
+cvs_id_string="$Id: xpak.py,v 1.13.2.2 2005/01/16 02:35:33 carpaski Exp $"[5:-2]
 
 # The format for a tbz2/xpak:
 #
@@ -56,6 +56,8 @@ def xpak(rootdir,outfile=None):
 	xpak segment."""
 	try:
 		origdir=os.getcwd()
+	except SystemExit, e:
+		raise
 	except:
 		os.chdir("/")
 		origdir="/"
@@ -194,6 +196,8 @@ def xpand(myid,mydest):
 	mydata=myid[1]
 	try:
 		origdir=os.getcwd()
+	except SystemExit, e:
+		raise
 	except:
 		os.chdir("/")
 		origdir="/"
@@ -236,6 +240,7 @@ class tbz2:
 		if not self.scan():
 			raise IOError
 		if cleanup and os.path.exists(datadir):
+			# XXX: Potentially bad
 			os.system("rm -Rf "+datadir+"/*")
 		if not os.path.exists(datadir):
 			os.makedirs(datadir)
@@ -258,6 +263,7 @@ class tbz2:
 		myfile.flush()
 		myfile.close()
 		if cleanup:
+			# XXX: Potentially bad
 			os.system("rm -Rf "+datadir)
 		return 1
 
@@ -299,6 +305,8 @@ class tbz2:
 			self.datapos=a.tell()
 			a.close()
 			return 2
+		except SystemExit, e:
+			raise
 		except:
 			return 0
 
@@ -334,6 +342,8 @@ class tbz2:
 			return 0
 		try:
 			origdir=os.getcwd()
+		except SystemExit, e:
+			raise
 		except:
 			os.chdir("/")
 			origdir="/"
