@@ -25,7 +25,7 @@
  *  as some of the InstallWatch code was used.
  *
  *
- *  $Header: /home/cvsroot/gentoo-src/portage/src/sandbox-1.1/libsandbox.c,v 1.16 2004/03/22 01:40:58 carpaski Exp $
+ *  $Header: /var/cvsroot/gentoo-src/portage/src/sandbox-1.1/Attic/libsandbox.c,v 1.22.2.3 2004/12/01 22:14:09 carpaski Exp $
  *
  */
 
@@ -114,7 +114,8 @@
 }
 
 static char sandbox_lib[255];
-static char sandbox_pids_file[255];
+//static char sandbox_pids_file[255];
+static char *sandbox_pids_file;
 
 typedef struct {
 	int show_access_violation;
@@ -246,6 +247,12 @@ init_wrappers(void)
 	true_execve = dlsym(libc_handle, "execve");
 }
 
+void
+_fini(void)
+{
+    free(sandbox_pids_file);
+}
+
 void
 _init(void)
 {
@@ -260,17 +267,13 @@ _init(void)
 
 	/* Get the path and name to this library */
 	tmp_string = get_sandbox_lib("/");
-	strncpy(sandbox_lib, tmp_string, 254);
+	strncpy(sandbox_lib, tmp_string, sizeof(sandbox_lib)-1);
 	if (tmp_string)
 		free(tmp_string);
 	tmp_string = NULL;
 
 	/* Generate sandbox pids-file path */
-	tmp_string = get_sandbox_pids_file();
-	strncpy(sandbox_pids_file, tmp_string, 254);
-	if (tmp_string)
-		free(tmp_string);
-	tmp_string = NULL;
+	sandbox_pids_file = get_sandbox_pids_file();
 
 	errno = old_errno;
 }
@@ -431,7 +434,6 @@ fopen(const char *pathname, const char *mode)
 int
 lchown(const char *path, uid_t owner, gid_t group)
 {
-/* Linux specific? */
 	int result = -1;
 	char canonic[SB_PATH_MAX];
 
@@ -439,8 +441,8 @@ lchown(const char *path, uid_t owner, gid_t group)
 
 	if FUNCTION_SANDBOX_SAFE
 		("lchown", canonic) {
-		check_dlsym(chown);
-		result = true_chown(path, owner, group);
+		check_dlsym(lchown);
+		result = true_lchown(path, owner, group);
 		}
 
 	return result;
@@ -466,6 +468,7 @@ link(const char *oldpath, const char *newpath)
 
 int
 mkdir(const char *pathname, mode_t mode)
+// returns 0 success, or -1 if an error occurred
 {
 	int result = -1, my_errno = errno;
 	char canonic[SB_PATH_MAX];
@@ -476,7 +479,7 @@ mkdir(const char *pathname, mode_t mode)
 	/* Check if the directory exist, return EEXIST rather than failing */
 	if (0 == lstat(canonic, &st)) {
 		errno = EEXIST;
-		return errno;
+		return -1; 
 	}
 	errno = my_errno;
 
@@ -738,6 +741,7 @@ execve(const char *filename, char *const argv[], char *const envp[])
 	int env_len = 0;
 	char canonic[SB_PATH_MAX];
 	char **my_env = NULL;
+	int kill_env = 1;
 	/* We limit the size LD_PRELOAD can be here, but it should be enough */
 	char tmp_str[4096];
 
@@ -749,6 +753,7 @@ execve(const char *filename, char *const argv[], char *const envp[])
 			if (strstr(envp[count], "LD_PRELOAD=") == envp[count]) {
 				if (NULL != strstr(envp[count], sandbox_lib)) {
 					my_env = (char **) envp;
+					kill_env = 0;
 					break;
 				} else {
 					int i = 0;
@@ -808,7 +813,7 @@ execve(const char *filename, char *const argv[], char *const envp[])
 		result = true_execve(filename, argv, my_env);
 		old_errno = errno;
 
-		if (my_env) {
+		if (my_env && kill_env) {
 			free(my_env);
 			my_env = NULL;
 		}
@@ -1169,6 +1174,7 @@ check_syscall(sbcontext_t * sbcontext, const char *func, const char *file)
 	char *debug_log_path = NULL;
 	int debug_log_file = 0;
 	char buffer[512];
+	char *dpath = NULL;
 
 	init_wrappers();
 
@@ -1178,10 +1184,8 @@ check_syscall(sbcontext_t * sbcontext, const char *func, const char *file)
 	} else {
 		tmp_buffer = (char *) malloc(SB_PATH_MAX * sizeof (char));
 		egetcwd(tmp_buffer, SB_PATH_MAX - 1);
-		absolute_path = (char *) malloc((strlen(tmp_buffer) + 1 + strlen(file) + 1)
-																		* sizeof (char));
+		absolute_path = (char *) malloc((strlen(tmp_buffer) + 1 + strlen(file) + 1) * sizeof (char));
 		sprintf(absolute_path, "%s/%s", tmp_buffer, file);
-
 		if (tmp_buffer)
 			free(tmp_buffer);
 		tmp_buffer = NULL;
@@ -1206,23 +1210,30 @@ check_syscall(sbcontext_t * sbcontext, const char *func, const char *file)
 			if (NULL != log_path) {
 				sprintf(buffer, "%s:%*s%s\n", func, (int) (10 - strlen(func)), "",
 								absolute_path);
-
+				// log_path somehow gets corrupted.  figuring out why would be good.
+				dpath = strdup(log_path);
 				if ((0 == lstat(log_path, &log_stat))
 						&& (0 == S_ISREG(log_stat.st_mode))
 						) {
 					fprintf(stderr,
-									"\e[31;01mSECURITY BREACH\033[0m  %s already exists and is not a regular file.\n",
-									log_path);
+						"\e[31;01mSECURITY BREACH\033[0m  %s already exists and is not a regular file.\n",
+						dpath);
+				} else if (0 == check_access(sbcontext, "open_wr", dpath)) {
+					unsetenv("SANDBOX_LOG");
+					fprintf(stderr,
+						"\e[31;01mSECURITY BREACH\033[0m SANDBOX_LOG %s isn't allowed via SANDBOX_WRITE\n",
+						dpath);
 				} else {
-					log_file = true_open(log_path,
-															 O_APPEND | O_WRONLY
-															 | O_CREAT,
-															 S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
+					log_file = true_open(dpath,
+						 O_APPEND | O_WRONLY
+						 | O_CREAT,
+						 S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
 					if (log_file >= 0) {
 						write(log_file, buffer, strlen(buffer));
 						close(log_file);
 					}
 				}
+				free(dpath);
 			}
 		}
 
@@ -1232,28 +1243,36 @@ check_syscall(sbcontext_t * sbcontext, const char *func, const char *file)
 			if (0 != strncmp(absolute_path, debug_log_path, strlen(debug_log_path))) {
 				sprintf(buffer, "%s:%*s%s\n", func, (int) (10 - strlen(func)), "",
 								absolute_path);
-
+				//debug_log_path somehow gets corupted, same thing as log_path above.
+				dpath = strdup(debug_log_path);
 				if ((0 == lstat(debug_log_path, &debug_log_stat))
 						&& (0 == S_ISREG(debug_log_stat.st_mode))
 						) {
 					fprintf(stderr,
-									"\e[31;01mSECURITY BREACH\033[0m  %s already exists and is not a regular file.\n",
-									log_path);
-				} else {
+						"\e[31;01mSECURITY BREACH\033[0m  %s already exists and is not a regular file.\n",
+						debug_log_path);
+				} else if (0 == check_access(sbcontext, "open_wr", dpath)) {
+					unsetenv("SANDBOX_DEBUG");
+					unsetenv("SANDBOX_DEBUG_LOG");
+					fprintf(stderr,
+						"\e[31;01mSECURITY BREACH\033[0m  SANDBOX_DEBUG_LOG %s isn't allowed by SANDBOX_WRITE.\n",
+						dpath);
+				} else {					
 					debug_log_file =
-							true_open(debug_log_path,
-												O_APPEND | O_WRONLY |
-												O_CREAT, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
+						true_open(dpath,
+							O_APPEND | O_WRONLY |
+							O_CREAT, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
 					if (debug_log_file >= 0) {
 						write(debug_log_file, buffer, strlen(buffer));
 						close(debug_log_file);
 					}
 				}
+				free(dpath);
 			}
 		} else {
 			fprintf(stderr,
-							"\e[32;01mACCESS ALLOWED\033[0m %s:%*s%s\n",
-							func, (int) (10 - strlen(func)), "", absolute_path);
+				"\e[32;01mACCESS ALLOWED\033[0m %s:%*s%s\n",
+				func, (int) (10 - strlen(func)), "", absolute_path);
 		}
 	}
 
@@ -1349,8 +1368,9 @@ before_syscall_open_int(const char *func, const char *file, int flags)
 static int
 before_syscall_open_char(const char *func, const char *file, const char *mode)
 {
-	if ((strcmp(mode, "r") == 0) || (strcmp(mode, "rb") == 0)
-			|| (strcmp(mode, "rm") == 0)) {
+	if (*mode == 'r' && ((strcmp(mode, "r") == 0) ||
+			     /* The strspn accept args are known non-writable modifiers */
+			     (strlen(++mode) == strspn(mode, "xbtmc")))) {
 		return before_syscall("open_rd", file);
 	} else {
 		return before_syscall("open_wr", file);
