@@ -11,10 +11,10 @@
 **    Copyright (C) 2001 Geert Bevin, Uwyn, http://www.uwyn.com
 **    Distributed under the terms of the GNU General Public License, v2 or later 
 **    Author : Geert Bevin <gbevin@uwyn.com>
-**  $Header: /home/cvsroot/gentoo-src/portage/src/sandbox-1.1/sandbox.c,v 1.13 2004/01/12 21:11:46 carpaski Exp $
+**  $Header: /var/cvsroot/gentoo-src/portage/src/sandbox-1.1/Attic/sandbox.c,v 1.20.2.1 2004/12/01 22:14:09 carpaski Exp $
 */
 
-#define _GNU_SOURCE
+/* #define _GNU_SOURCE */
 
 #include <errno.h>
 #include <fcntl.h>
@@ -36,6 +36,7 @@
 int preload_adaptable = 1;
 int cleaned_up = 0;
 int print_debug = 0;
+int stop_called = 0;
 
 /* Read pids file, and load active pids into an array.  Return number of pids in array */
 int
@@ -160,8 +161,7 @@ cleanup()
 	int pids_file = -1, num_of_pids = 0;
 	int *pids_array = NULL;
 	char pid_string[255];
-	char sandbox_pids_file[255];
-	char *tmp_string;
+	char *sandbox_pids_file;
 #ifdef USE_LD_SO_PRELOAD
 	int preload_file = -1, num_of_preloads = 0;
 	char preload_entry[255];
@@ -169,11 +169,7 @@ cleanup()
 #endif
 
 	/* Generate sandbox pids-file path */
-	tmp_string = get_sandbox_pids_file();
-	strncpy(sandbox_pids_file, tmp_string, 254);
-	if (tmp_string)
-		free(tmp_string);
-	tmp_string = NULL;
+	sandbox_pids_file = get_sandbox_pids_file();
 
 	/* Remove this sandbox's bash pid from the global pids
 	 * file if it has rights to adapt the ld.so.preload file */
@@ -186,14 +182,14 @@ cleanup()
 
 		/* Stat the PIDs file, make sure it exists and is a regular file */
 		if (file_exist(sandbox_pids_file, 1) <= 0) {
-			perror(">>> pids file is not a regular file");
+			fprintf(stderr, ">>> pids file is not a regular file");
 			success = 0;
 			/* We should really not fail if the pidsfile is missing here, but
 			 * rather just exit cleanly, as there is still some cleanup to do */
 			return;
 		}
 
-		pids_file = file_open(sandbox_pids_file, "r+", 1, 0664);
+		pids_file = file_open(sandbox_pids_file, "r+", 1, 0664, "portage");
 		if (-1 == pids_file) {
 			success = 0;
 			/* Nothing more to do here */
@@ -213,7 +209,7 @@ cleanup()
 			if (print_debug)
 				printf("Cleaning up /etc/ld.so.preload.\n");
 
-			preload_file = file_open("/etc/ld.so.preload", "r+", 1, 0664);
+			preload_file = file_open("/etc/ld.so.preload", "r+", 1, 0644);
 			if (-1 != preload_file) {
 				/* Load all the preload libraries into an array */
 				num_of_preloads = load_preload_libs(preload_file, &preload_array);
@@ -285,6 +281,7 @@ cleanup()
 		pids_array = NULL;
 	}
 
+	free(sandbox_pids_file);
 	if (0 == success)
 		return;
 }
@@ -292,102 +289,75 @@ cleanup()
 void
 stop(int signum)
 {
-	printf("Caught signal %d\r\n", signum);
+	if (stop_called == 0) {
+		stop_called = 1;
+		printf("Caught signal %d in pid %d\r\n", signum, getpid());
 	cleanup();
+	} else {
+		fprintf(stderr, "Pid %d alreadly caught signal and is still cleaning up\n", getpid());
+	}
 }
 
 void
 setenv_sandbox_write(char *home_dir, char *portage_tmp_dir, char *var_tmp_dir,
 										 char *tmp_dir)
 {
-	char sandbox_write_var[1024];
+	char buf[1024];
+	
+	/* bzero out entire buffer then append trailing 0 */
+	memset(buf, 0, sizeof(buf));
 
 	if (!getenv(ENV_SANDBOX_WRITE)) {
-		/* these should go into make.globals later on */
-		strcpy(sandbox_write_var, "");
-		strcat(sandbox_write_var,
-					 "/dev/zero:/dev/fd/:/dev/null:/dev/pts/:/dev/vc/:/dev/tty:/tmp/");
-		strcat(sandbox_write_var, ":");
-		/* NGPT support */
-		strcat(sandbox_write_var, "/dev/shm/ngpt");
-		strcat(sandbox_write_var, ":");
-		strcat(sandbox_write_var, "/var/log/scrollkeeper.log");
-		strcat(sandbox_write_var, ":");
-		strcat(sandbox_write_var, home_dir);
-		strcat(sandbox_write_var, "/.gconfd/lock");
-		strcat(sandbox_write_var, ":");
-		strcat(sandbox_write_var, home_dir);
-		strcat(sandbox_write_var, "/.bash_history");
-		strcat(sandbox_write_var, ":");
-		strcat(sandbox_write_var, "/usr/tmp/conftest");
-		strcat(sandbox_write_var, ":");
-		strcat(sandbox_write_var, "/usr/lib/conftest");
-		strcat(sandbox_write_var, ":");
-		strcat(sandbox_write_var, "/usr/tmp/cf");
-		strcat(sandbox_write_var, ":");
-		strcat(sandbox_write_var, "/usr/lib/cf");
-		strcat(sandbox_write_var, ":");
+		/* these could go into make.globals later on */
+		snprintf(buf, sizeof(buf),
+			"%s:%s/.gconfd/lock:%s/.bash_history:",		\
+			"/dev/zero:/dev/fd/:/dev/null:/dev/pts/:"	\
+			"/dev/vc/:/dev/tty:/tmp/:"			\
+			"/dev/shm/ngpt:/var/log/scrollkeeper.log:"	\
+			"/usr/tmp/conftest:/usr/lib/conftest:"		\
+			"/usr/lib32/conftest:/usr/lib64/conftest:"	\
+			"/usr/tmp/cf:/usr/lib/cf:/usr/lib32/cf:/usr/lib64/cf",
+			home_dir, home_dir);
+
 		if (NULL == portage_tmp_dir) {
-			strcat(sandbox_write_var, tmp_dir);
-			strcat(sandbox_write_var, ":");
-			strcat(sandbox_write_var, var_tmp_dir);
-			strcat(sandbox_write_var, ":");
-			strcat(sandbox_write_var, "/tmp/");
-			strcat(sandbox_write_var, ":");
-			strcat(sandbox_write_var, "/var/tmp/");
-
-			/* How the heck is this possible?? we just set it above! */
-		} else if (0 == strcmp(sandbox_write_var, "/var/tmp/")) {
-			strcat(sandbox_write_var, portage_tmp_dir);
-			strcat(sandbox_write_var, ":");
-			strcat(sandbox_write_var, tmp_dir);
-			strcat(sandbox_write_var, ":");
-			strcat(sandbox_write_var, "/tmp/");
-
-			/* Still don't think this is possible, am I just stupid or something? */
-		} else if (0 == strcmp(sandbox_write_var, "/tmp/")) {
-			strcat(sandbox_write_var, portage_tmp_dir);
-			strcat(sandbox_write_var, ":");
-			strcat(sandbox_write_var, var_tmp_dir);
-			strcat(sandbox_write_var, ":");
-			strcat(sandbox_write_var, "/var/tmp/");
-
-			/* Amazing, one I think is possible */
+			strncat(buf, tmp_dir, sizeof(buf));
+			strncat(buf, ":", sizeof(buf));
+			strncat(buf, var_tmp_dir, sizeof(buf));
+			strncat(buf, ":/tmp/:/var/tmp/", sizeof(buf));
 		} else {
-			strcat(sandbox_write_var, portage_tmp_dir);
-			strcat(sandbox_write_var, ":");
-			strcat(sandbox_write_var, tmp_dir);
-			strcat(sandbox_write_var, ":");
-			strcat(sandbox_write_var, var_tmp_dir);
-			strcat(sandbox_write_var, ":");
-			strcat(sandbox_write_var, "/tmp/");
-			strcat(sandbox_write_var, ":");
-			strcat(sandbox_write_var, "/var/tmp/");
+			strncat(buf, portage_tmp_dir, sizeof(buf));
+			strncat(buf, ":", sizeof(buf));
+			strncat(buf, tmp_dir, sizeof(buf));
+			strncat(buf, ":", sizeof(buf));
+			strncat(buf, var_tmp_dir, sizeof(buf));
+			strncat(buf, ":/tmp/:/var/tmp/", sizeof(buf));
 		}
-
-		setenv(ENV_SANDBOX_WRITE, sandbox_write_var, 1);
+		buf[sizeof(buf) - 1] = '\0';
+		setenv(ENV_SANDBOX_WRITE, buf, 1);
 	}
 }
 
 void
 setenv_sandbox_predict(char *home_dir)
 {
-	char sandbox_predict_var[1024];
+	char buf[1024];
+
+	memset(buf, 0, sizeof(buf));
 
 	if (!getenv(ENV_SANDBOX_PREDICT)) {
 		/* these should go into make.globals later on */
-		strcpy(sandbox_predict_var, "");
-		strcat(sandbox_predict_var, home_dir);
-		strcat(sandbox_predict_var, "/.");
-		strcat(sandbox_predict_var, ":");
-		strcat(sandbox_predict_var, "/usr/lib/python2.0/");
-		strcat(sandbox_predict_var, ":");
-		strcat(sandbox_predict_var, "/usr/lib/python2.1/");
-		strcat(sandbox_predict_var, ":");
-		strcat(sandbox_predict_var, "/usr/lib/python2.2/");
-		strcat(sandbox_predict_var, ":");
-		strcat(sandbox_predict_var, "/usr/lib/python2.3/");
-		setenv(ENV_SANDBOX_PREDICT, sandbox_predict_var, 1);
+		snprintf(buf, sizeof(buf), "%s/.:"	\
+				"/usr/lib/python2.0/:"	\
+				"/usr/lib/python2.1/:"	\
+				"/usr/lib/python2.2/:"	\
+				"/usr/lib/python2.3/:"	\
+				"/usr/lib/python2.4/:"	\
+				"/usr/lib/python2.5/:"	\
+				"/usr/lib/python3.0/:",
+			home_dir);
+
+		buf[sizeof(buf) - 1] = '\0';
+		setenv(ENV_SANDBOX_PREDICT, buf, 1);
 	}
 }
 
@@ -396,11 +366,11 @@ print_sandbox_log(char *sandbox_log)
 {
 	int sandbox_log_file = -1;
 	char *beep_count_env = NULL;
-	int i, beep_count = 0;
+	int i, color, beep_count = 0;
 	long len = 0;
 	char *buffer = NULL;
 
-	sandbox_log_file = file_open(sandbox_log, "r", 1, 0664);
+	sandbox_log_file = file_open(sandbox_log, "r", 1, 0664, "portage");
 	if (-1 == sandbox_log_file)
 		return 0;
 
@@ -410,10 +380,15 @@ print_sandbox_log(char *sandbox_log)
 	read(sandbox_log_file, buffer, len);
 	file_close(sandbox_log_file);
 
-	printf
-			("\e[31;01m--------------------------- ACCESS VIOLATION SUMMARY ---------------------------\033[0m\n");
-	printf("\e[31;01mLOG FILE = \"%s\"\033[0m\n", sandbox_log);
-	printf("\n");
+	color = ( (getenv("NOCOLOR") != NULL) ? 0 : 1);
+
+	if (color) printf("\e[31;01m");
+	printf("--------------------------- ACCESS VIOLATION SUMMARY ---------------------------");
+	if (color) printf("\033[0m");
+	if (color) printf("\e[31;01m");
+	printf("\nLOG FILE = \"%s\"", sandbox_log);
+	if (color) printf("\033[0m");
+	printf("\n\n");
 	printf("%s", buffer);
 	if (buffer)
 		free(buffer);
@@ -520,7 +495,7 @@ main(int argc, char **argv)
 	char sandbox_debug_log[255];
 	char sandbox_dir[255];
 	char sandbox_lib[255];
-	char sandbox_pids_file[255];
+	char *sandbox_pids_file;
 	char sandbox_rc[255];
 	char pid_string[255];
 	char **argv_bash = NULL;
@@ -568,11 +543,7 @@ main(int argc, char **argv)
 		tmp_string = NULL;
 
 		/* Generate sandbox pids-file path */
-		tmp_string = get_sandbox_pids_file();
-		strncpy(sandbox_pids_file, tmp_string, 254);
-		if (tmp_string)
-			free(tmp_string);
-		tmp_string = NULL;
+		sandbox_pids_file = get_sandbox_pids_file();
 
 		/* Generate sandbox bashrc path */
 		tmp_string = get_sandbox_rc(sandbox_dir);
@@ -585,11 +556,14 @@ main(int argc, char **argv)
 		if (print_debug)
 			printf("Verification of the required files.\n");
 
+#ifndef SB_HAVE_64BIT_ARCH
 		if (file_exist(sandbox_lib, 0) <= 0) {
 			fprintf(stderr, "Could not open the sandbox library at '%s'.\n",
 							sandbox_lib);
 			return -1;
-		} else if (file_exist(sandbox_rc, 0) <= 0) {
+		} 
+#endif
+		if (file_exist(sandbox_rc, 0) <= 0) {
 			fprintf(stderr, "Could not open the sandbox rc file at '%s'.\n",
 							sandbox_rc);
 			return -1;
@@ -607,7 +581,7 @@ main(int argc, char **argv)
 
 		if (getuid() == 0) {
 			/* Our r+ also will create the file if it doesn't exist */
-			preload_file = file_open("/etc/ld.so.preload", "r+", 1, 0664);
+			preload_file = file_open("/etc/ld.so.preload", "r+", 1, 0644);
 			if (-1 == preload_file) {
 				preload_adaptable = 0;
 /*      exit(1);*/
@@ -684,7 +658,7 @@ main(int argc, char **argv)
 
 		setenv(ENV_SANDBOX_LOG, sandbox_log, 1);
 
-		snprintf(sandbox_debug_log, 254, "%s%s%s",
+		snprintf(sandbox_debug_log, sizeof(sandbox_debug_log), "%s%s%s",
 						 DEBUG_LOG_FILE_PREFIX, pid_string, LOG_FILE_EXT);
 		setenv(ENV_SANDBOX_DEBUG_LOG, sandbox_debug_log, 1);
 
@@ -699,7 +673,12 @@ main(int argc, char **argv)
 		 * this, access is denied to /var/tmp, hurtin' ebuilds.
 		 */
 
-		realpath(getenv("PORTAGE_TMPDIR"), portage_tmp_dir);
+		{	char *e;
+			e = getenv("PORTAGE_TMPDIR");
+			if ( e && ( strlen(e) < sizeof(portage_tmp_dir)-1 ) && (strlen(e) > 1) )
+				realpath(e, portage_tmp_dir);
+
+		}
 		realpath("/var/tmp", var_tmp_dir);
 		realpath("/tmp", tmp_dir);
 
@@ -755,28 +734,6 @@ main(int argc, char **argv)
 				strcat(argv_bash[4], argv[i]);
 			}
 		}
-#if 0
-		char *argv_bash[] = {
-			"/bin/bash",
-			"-rcfile",
-			NULL,
-			NULL,
-			NULL,
-			NULL
-		};
-
-		/* adding additional bash arguments */
-		for (i = 1; i < argc; i++) {
-			if (1 == i) {
-				argv_bash[3] = run_str;
-				argv_bash[4] = run_arg;
-				strcpy(argv_bash[4], argv[i]);
-			} else {
-				strcat(argv_bash[4], " ");
-				strcat(argv_bash[4], argv[i]);
-			}
-		}
-#endif
 
 		/* set up the required signal handlers */
 		signal(SIGHUP, &stop);
@@ -797,9 +754,9 @@ main(int argc, char **argv)
 		success = 1;
 		if (file_exist(sandbox_pids_file, 1) < 0) {
 			success = 0;
-			fprintf(stderr, ">>> %s is not a regular file", sandbox_pids_file);
+			fprintf(stderr, ">>> %s is not a regular file\n", sandbox_pids_file);
 		} else {
-			pids_file = file_open(sandbox_pids_file, "r+", 1, 0664);
+			pids_file = file_open(sandbox_pids_file, "r+", 1, 0664, "portage");
 			if (-1 == pids_file)
 				success = 0;
 		}
